//
//  ViewController.m
//  SpeedTestExample
//
//  Created by dusmit on 2020/8/31.
//  Copyright © 2020 巨擎网络科技(济南)有限公司. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

@property (nonatomic, strong) SpeedTest *speed;
@property (nonatomic, weak) UIButton *speedTestButton;
@property (nonatomic, weak) UILabel *msgLabel;
@property (nonatomic, weak) UILabel *downloadSpeedLabel;
@property (nonatomic, weak) UILabel *uploadSpeedLabel;
@property (nonatomic, weak) UILabel *respeedLabel;
@property (nonatomic, weak) UITextField *thresholdTextField;
@property (nonatomic, weak) UISwitch *fastSpeedSwitch;
@property (nonatomic, weak) UISwitch *autoSwitch;
@property (nonatomic, weak) UIActivityIndicatorView *activityIndicator;
@property (nonatomic, assign) NSInteger repeatCount;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = UIColor.whiteColor;
    self.speed = [[SpeedTest alloc] init];
    self.speed.speedUnit = SpeedUnitKBps;
    [self createMainUI];
}

- (void)createMainUI {
    
    NSArray *titleArray = @[@"SpeedTest", @"Cancel", @"Speed View"];
    for (int i = 0; i < titleArray.count; i ++) {
        
        UIButton *button = [UIButton buttonWithType: UIButtonTypeCustom];
        [button setTitle: titleArray[i] forState: UIControlStateNormal];
        [button setTitleColor: UIColor.redColor forState: UIControlStateNormal];
        [button.titleLabel setFont: [UIFont systemFontOfSize:15]];
        switch (i) {
            case 0:
                self.speedTestButton = button;
                [button addTarget: self action: @selector(speedTest:) forControlEvents: UIControlEventTouchUpInside];
            break;
            case 1:
                [button addTarget: self action: @selector(cancel) forControlEvents: UIControlEventTouchUpInside];
                break;
            case 2:
                [button addTarget: self action: @selector(speedButton) forControlEvents: UIControlEventTouchUpInside];
            break;
        }
        [button setFrame: CGRectMake(20+((self.view.frame.size.width-80)/3+20)*i, 120, (self.view.frame.size.width-80)/3, 40)];
        button.layer.cornerRadius = 20;
        button.layer.borderWidth = 1.0f;
        button.layer.borderColor = UIColor.redColor.CGColor;
        [self.view addSubview:button];
    }
    
    NSArray *switchArray = @[@"阈值:", @"下载结果小于阈值是否切换节点自动重新开始测速:", @"是否启用快速测速/省流量测速:"];
    for (int i = 0; i < switchArray.count; i ++) {
        
        UILabel *switchLabel = [[UILabel alloc] initWithFrame: CGRectMake(20, 200+60*i, self.view.frame.size.width-40, 40)];
        switchLabel.text = switchArray[i];
        switchLabel.textColor = UIColor.blackColor;
        switchLabel.font = [UIFont systemFontOfSize: 12];
        [self.view addSubview: switchLabel];
        
        if (i == 0) {
            UITextField *textField = [[UITextField alloc] initWithFrame: CGRectMake(self.view.frame.size.width-100, 200+60*i, 80, 40)];
            textField.backgroundColor = UIColor.whiteColor;
            textField.textColor = UIColor.blackColor;
            textField.textAlignment = NSTextAlignmentRight;
            textField.keyboardType = UIKeyboardTypeNumberPad;
            textField.layer.borderWidth = 0.5;
            textField.layer.borderColor = UIColor.grayColor.CGColor;
            textField.layer.cornerRadius = 5.0;
            textField.layer.masksToBounds = true;
            self.thresholdTextField = textField;
            [self.view addSubview: textField];
        } else {
            UISwitch *changeSwitch = [[UISwitch alloc] init];
            changeSwitch.frame = CGRectMake(self.view.frame.size.width-changeSwitch.frame.size.width-20, switchLabel.frame.origin.y+20-changeSwitch.frame.size.height/2.0, 0, 0);
            if (i == 1) {
                self.autoSwitch = changeSwitch;
            } else {
                self.fastSpeedSwitch = changeSwitch;
            }
            [self.view addSubview: changeSwitch];
        }
    }
    
    UILabel *msgLabel = [[UILabel alloc] initWithFrame: CGRectMake(20, 400, self.view.frame.size.width-40, 20)];
    msgLabel.text = @"ping: 0, jitter: 0, packetLoss: 0";
    msgLabel.textAlignment = NSTextAlignmentCenter;
    self.msgLabel = msgLabel;
    [self.view addSubview: msgLabel];
    
    UILabel *downloadSpeedLabel = [[UILabel alloc] initWithFrame: CGRectMake(20, 450, self.view.frame.size.width/2.0-20, 20)];
    downloadSpeedLabel.text = @"下载速度：0";
    downloadSpeedLabel.textAlignment = NSTextAlignmentCenter;
    self.downloadSpeedLabel = downloadSpeedLabel;
    [self.view addSubview: downloadSpeedLabel];
    
    UILabel *uploadSpeedLabel = [[UILabel alloc] initWithFrame: CGRectMake(self.view.frame.size.width/2.0, downloadSpeedLabel.frame.origin.y, self.view.frame.size.width/2.0-20, 20)];
    uploadSpeedLabel.text = @"上传速度：0";
    uploadSpeedLabel.textAlignment = NSTextAlignmentCenter;
    self.uploadSpeedLabel = uploadSpeedLabel;
    [self.view addSubview: uploadSpeedLabel];
    
    UILabel *respeedLabel = [[UILabel alloc] initWithFrame: CGRectMake(20, 600, self.view.frame.size.width-40, 20)];
    respeedLabel.text = @"自动下载次数：0";
    respeedLabel.font = [UIFont systemFontOfSize: 12];
    self.respeedLabel = respeedLabel;
    [self.view addSubview: respeedLabel];
    
    UIActivityIndicatorView *activityIndicator = [[UIActivityIndicatorView alloc] init];
    activityIndicator.hidesWhenStopped = true;
    activityIndicator.center = self.view.center;
    self.activityIndicator = activityIndicator;
    [self.view addSubview: activityIndicator];
}

- (void)speedTest: (UIButton *)button {

    [self.activityIndicator startAnimating];
    [button setEnabled:false];
    self.repeatCount = 0;
    self.respeedLabel.text = @"自动下载次数：0";

    self.speed.speedUnit = SpeedUnitMbps;
    self.speed.fastSpeed = self.fastSpeedSwitch.on;
    self.speed.autoSwitchNode = self.autoSwitch.on;
    self.speed.threshold = [NSString stringWithFormat:@"%@", self.thresholdTextField.text].floatValue;
    
    __weak typeof(self) weakSelf = self;
    [self.speed speedTestWithPingResult:^(float ping, float jitter, float packetLoss) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSLog(@"ping:%.1f, jitter:%.1f, packetLoss:%.1f", ping, jitter, packetLoss);
            weakSelf.msgLabel.text = [NSString stringWithFormat:@"ping: %.1f, jitter: %.1f, packetLoss: %.1f", ping, jitter, packetLoss];
            weakSelf.respeedLabel.text = [NSString stringWithFormat:@"自动下载次数：%ld", (long)weakSelf.repeatCount];
            weakSelf.repeatCount ++;
            [weakSelf.activityIndicator stopAnimating];
        });
    } speed:^(NSInteger type, float speed) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSLog(@"speed-- type: %ld, speed: %f", (long)type, speed);
            if (type == SpeedTypeDownload) {
                weakSelf.downloadSpeedLabel.text = [NSString stringWithFormat:@"下载速度：%.2f", speed];
            } else {
                weakSelf.uploadSpeedLabel.text = [NSString stringWithFormat:@"上传速度：%.2f", speed];
            }
        });
    } finish:^(NSInteger type, float speed, int64_t flow) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSLog(@"finish-- type: %ld, speed: %f", (long)type, speed);
            if (type == SpeedTypeUpload) {
                weakSelf.downloadSpeedLabel.text = @"下载速度：0";
                weakSelf.uploadSpeedLabel.text = @"上传速度：0";
                [button setEnabled: true];
            }
        });
    } error:^(NSError * _Nonnull error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSLog(@"---***%@", error);
            [button setEnabled: true];
            [weakSelf.activityIndicator stopAnimating];
        });
    }];
}

- (void)cancel {
    
    [self.speed stop];
    self.downloadSpeedLabel.text = @"下载速度：0";
    self.uploadSpeedLabel.text = @"上传速度：0";
    [self.speedTestButton setEnabled: true];
}

- (void)speedButton {

    if ([SpeedTestConfig isRegisterSuccess]) {
        [SpeedTestConfig pushToSpeedViewControllerWithNavigationController:self.navigationController unit:SpeedUnitMBps isAutoSpeed:false];
    } else {

    }
}

- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    
    [self.view endEditing: true];
}

- (void)dealloc {
    
    [self cancel];
    NSLog(@"dealloc");
}


@end
